package at.oefai.aaa.agent.jam.types;

import java.io.Serializable;

/**
 * Represents a particular variable binding.
 * @author Marc Huber
 * @author Jaeho Lee
 */
public class BindingValue implements Serializable {
    // Binding value can be either a constant value
    private Value value = null;
    // or a variable defined in other external binding
    private Variable externalVariable = null;
    private Binding externalBinding = null;

    /**
     * value constructor.
     * @param pValue the value the variable(s) should be bound to
     */
    public BindingValue(final Value pValue) {
        this.value = pValue;
    }

    /**
     * external binding constructor.
     * @param extVar the external variable to be bound too
     * @param extBind and the binding to be used
     */
    public BindingValue(final Variable extVar, final Binding extBind) {
        this.externalVariable = extVar;
        this.externalBinding = extBind;
    }


    /** Copy constructor. */
    public BindingValue(final BindingValue bval) {
        this.externalVariable = bval.getExternalVariable();
        this.externalBinding = bval.getExternalBinding();
        this.value = bval.getValue();
        if (this.value != null) { // copy the value
            this.value = Value.newValue(this.value);
        }
    }

    public final Value getValue() { return this.value; }

    public final void setValue(final Value v) { this.value = v; }

    public final Variable getExternalVariable() { return this.externalVariable; }

    public final Binding getExternalBinding() { return this.externalBinding; }

    public final void setExternal(final Variable v, final Binding b) {
        this.externalVariable = v;
        this.externalBinding = b;
    }

}

