package at.oefai.aaa.agent.jam.types;

import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * Expression List.
 * @author Marc Huber
 * @author Jaeho Lee
 */
public class ExpList extends DList<Expression> implements Serializable {
    /** Default constructor. */
    public ExpList() {
        super();
    }

    /** Shallow copy constructor. */
    public ExpList(final ExpList pList) {
        super(pList);
    }


    /** Convert all variables elements of the expression into constants.
     *  create new copies of Values. */
    public ExpList explistEval(final Binding binding) {
        ExpList newExpList = new ExpList();
        for (Expression e : this) {
            newExpList.addLast(Value.newValue(e.eval(binding)));
        }
        return newExpList;
    }

    /** Return as a list of constants. Do not create new Value instances (!= explistEval) */
    public List<Value> asValueList(final Binding binding) {
        List<Value> valueList = new LinkedList<Value>();
        for (Expression e : this) {
            valueList.add(e.eval(binding));
        }
        return valueList;
    }

    /** Return true if all of the expressions in this list are defined values. */
    public boolean allDefined() {
        for (Expression e : this) {
            if (! (e instanceof Value && ((Value) e).isDefined())) {
                return false;
            }
        }
        return true;
    }

    /** Format the output and don't worry about being printed out in-line with other information. */
    public String verboseString(final Binding b) {
        StringBuffer sb = new StringBuffer();
        for (Expression e : this) {
            if (e instanceof ValueString) {
                sb.append("\"" + e + "\" ");
            } else {
                sb.append(e.eval(b) + " ");
            }
        }
        return sb.toString();
    }

    /** Format the output so that it's conducive to being printed out in-line with other information. */
    public String formattedString(final Binding b) {
        StringBuffer sb = new StringBuffer();
        // need to determine if there's a followup element, so no for-each loop possible
        for (Iterator<Expression> expIter = this.iterator(); expIter.hasNext();) {
            Expression e = expIter.next();
            sb.append(e.eval(b).formattedString(b));
            if (expIter.hasNext()) {
                sb.append(", ");
            }
        }
        return sb.toString();
    }

}


