package at.oefai.aaa.agent.jam.types;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

/**
 * a simple map that ensures unique variables in its context.
 * the variables are identified by name
 * @author Stefan Rank
 */
public class SymbolMap implements Serializable {

    private Map<String,Variable> symbols = new HashMap<String,Variable>();

    public synchronized int size() {
        return this.symbols.size();
    }

    /**
     * @return a set containing all the variable names
     */
    public synchronized Set<String> keySet() {
        return this.symbols.keySet();
    }

    /**
     * @param s the variable name
     * @return the variable (a newly created one if there was none of that name)
     */
    public synchronized Variable getVariable(final String s) {
        Variable v = this.symbols.get(s);
        if (v == null) {
            v = new Variable(this, s);
            this.symbols.put(s, v);
        }
        return v;
    }

}
