package at.oefai.aaa.agent.jam.types;

import java.io.Serializable;

/**
 * Represents a built-in JAM data-type.
 * @author Marc Huber
 * @author Jaeho Lee
 * @author Stefan Rank
 */
public class Value extends AbstractExpression implements Serializable {
    protected static final boolean DISABLE_ASSERT = true;
    public static final Value UNDEFINED = new Value();
    public static final Value TRUE      = Value.newValue(1);
    public static final Value FALSE     = Value.newValue(0);

    protected Value() {
        // default constructor only for undefined values and subtypes
    }

    // Factory methods for constructing Values:

    public static Value newValue(final Long l) {
        return new ValueLong(l.longValue());
    }

    public static Value newValue(final long l) {
        return new ValueLong(l);
    }

    public static Value newValue(final Integer i) {
        return new ValueLong(i.longValue());
    }

    public static Value newValue(final int l) {
        return new ValueLong(l);
    }

    public static Value newValue(final Double d) {
        return new ValueReal(d.doubleValue());
    }

    public static Value newValue(final double d) {
        return new ValueReal(d);
    }

    public static Value newValue(final String s) {
        return new ValueString(s);
    }

    public static Value newValue(final Object o) {
        if (o instanceof String) {
            return new ValueString((String) o);
        }
        return new ValueObject(o);
    }

    public static Value newValue(final Value v) {
        return v.copy();
    }

    /** cloning method: dont here as this is the base class (kind of a singleton for UNDEFINED). */
    Value copy() {
        return this;
    }

    // Required Member functions for Expression

    public Value eval(final Binding b) { return this; }

    // Member functions

    public boolean isTrue() { return false; }

    public boolean isDefined() { return this.getClass() != Value.class; }

    public long getLong() { return 0; }
    public double getReal() { return 0.0; }
    public String getString() { return null; }
    public Object getObject() { return null; }

    public String toString() { return "null"; }

    // Operators
    // (unfortunately Java does not consider the runtime types of arguments for
    //  determinig which overloaded method to call, so subclasses of Value have
    //  to explicitly check the runtime-types of the argument)

    public Value neg() { assert DISABLE_ASSERT : "negating Undefined Value"; return this; }

    public Value add(final Value v) { assert DISABLE_ASSERT : "adding Undefined Value"; return this; }
    public Value sub(final Value v) { assert DISABLE_ASSERT : "subtracting Undefined Value"; return this; }
    public Value mul(final Value v) { assert DISABLE_ASSERT : "multiplying Undefined Value"; return this; }
    public Value div(final Value v) { assert DISABLE_ASSERT : "dividing Undefined Value"; return this; }
    public Value mod(final Value v) { assert DISABLE_ASSERT : "modulo-ing Undefined Value"; return this; }

    public boolean not() { assert DISABLE_ASSERT : "not-ing Undefined Value"; return false; }

    public boolean lt(final Value v) { assert DISABLE_ASSERT : "comparing Undefined Value"; return false; }
    public boolean gt(final Value v) { assert DISABLE_ASSERT : "comparing Undefined Value"; return false; }
    public boolean le(final Value v) { assert DISABLE_ASSERT : "comparing Undefined Value"; return false; }
    public boolean ge(final Value v) { assert DISABLE_ASSERT : "comparing Undefined Value"; return false; }
    public boolean eq(final Value v) { assert DISABLE_ASSERT : "comparing Undefined Value"; return false; }
    public boolean ne(final Value v) { assert DISABLE_ASSERT : "comparing Undefined Value"; return false; }

    //

    public String verboseString(final Binding b) {
        return this.toString();
    }

    public String formattedString(final Binding b) {
        return this.toString();
    }
}

