package at.oefai.aaa.agent.jam.types;

import java.io.Serializable;

/**
 * Represents a built-in JAM Java Object data-type.
 * @author Marc Huber
 * @author Stefan Rank
 */
public class ValueObject extends Value implements Serializable {
    private Object value = null;

    ValueObject(final Object o) { this.value = o; }

    Value copy() { return new ValueObject(this.value); }

    // Member functions

    public boolean isTrue() { return this.value != null; }

    public long getLong() { return 0; } //?
    public double getReal() { return 0.0; } //?
    public String getString() { return this.toString(); }
    public Object getObject() { return this.value; }

    public String toString() { return (this.value == null) ? "null" : this.value.toString(); }

    // Operators

    public Value neg() { return this.copy(); } // no negate object !

    public Value add(final Value v) { assert DISABLE_ASSERT : "ValueObject " + this.value + "can't be added to"; return v; }
    public Value sub(final Value v) { assert DISABLE_ASSERT : "ValueObject " + this.value + "can't be subtracted from"; return v; }
    public Value mul(final Value v) { assert DISABLE_ASSERT : "ValueObject " + this.value + "can't be multiplied"; return v; }
    public Value div(final Value v) { assert DISABLE_ASSERT : "ValueObject " + this.value + "can't be divided"; return v; }
    public Value mod(final Value v) { assert DISABLE_ASSERT : "ValueObject " + this.value + "can't be modulo-ed"; return v; }

    public boolean not() { return ! isTrue(); }

    public boolean lt(final Value v) { assert DISABLE_ASSERT : "ValueObject " + this.value + "can't be compared";  return false; }
    public boolean gt(final Value v) { assert DISABLE_ASSERT : "ValueObject " + this.value + "can't be compared";  return false; }
    public boolean le(final Value v) { assert DISABLE_ASSERT : "ValueObject " + this.value + "can't be compared";  return false; }
    public boolean ge(final Value v) { assert DISABLE_ASSERT : "ValueObject " + this.value + "can't be compared";  return false; }
    public boolean eq(final Value v) {
        if (v instanceof ValueLong) { return this.internalEq((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalEq((ValueReal) v); }
        if (v instanceof ValueObject) { return this.internalEq((ValueObject) v); }
        return this.internalEq(v);
    }
    public boolean ne(final Value v) {
        if (v instanceof ValueLong) { return this.internalNe((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalNe((ValueReal) v); }
        if (v instanceof ValueObject) { return this.internalNe((ValueObject) v); }
        return this.internalNe(v);
    }

    // Internal Operators

    private boolean internalEq(final Value v) { assert DISABLE_ASSERT : "comparing Unknown Value type: " + v; return false; }
    private boolean internalEq(final ValueLong v) { return this.isTrue() == v.isTrue(); } // allows (obj == 0)
    private boolean internalEq(final ValueReal v) { return this.isTrue() == v.isTrue(); } // allows (obj == 0.0)
    private boolean internalEq(final ValueObject v) { return this.value == v.getObject(); }

    private boolean internalNe(final Value v) { assert DISABLE_ASSERT : "comparing Unknown Value type: " + v; return false; }
    private boolean internalNe(final ValueLong v) { return this.isTrue() != v.isTrue(); } // allows (obj != 0)
    private boolean internalNe(final ValueReal v) { return this.isTrue() != v.isTrue(); } // allows (obj != 0.0)
    private boolean internalNe(final ValueObject v) { return this.value != v.getObject(); }

}
