package at.oefai.aaa.agent.jam.types;

import java.io.Serializable;
import java.text.NumberFormat;

/**
 * Represents a built-in JAM Real data-type.
 * @author Marc Huber
 * @author Jaeho Lee
 */
public class ValueReal extends Value implements Serializable {
    private static final NumberFormat NF = NumberFormat.getNumberInstance();
    private double value = 0.0;

    ValueReal(final double d) { this.value = d; }

    ValueReal(final float f) { this.value = f; }

    Value copy() { return new ValueReal(this.value); }

    // Member functions

    // unsafe float compare, but boolean float should not be used anyway
    public boolean isTrue() { return this.value != 0.0; }

    public long getLong() { return (long) this.value; }
    public double getReal() { return this.value; }
    public String getString() { return this.toString(); }
    public Object getObject() { return new Double(this.value); }

    public String toString() { return NF.format(this.value); } //hopefully standard format is fixpoint

    // Operators

    public Value neg() { return new ValueReal(-this.value); }

    public Value add(final Value v) {
        if (v instanceof ValueLong) { return this.internalAdd((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalAdd((ValueReal) v); }
        if (v instanceof ValueString) { return this.internalAdd((ValueString) v); }
        return this.internalAdd(v);
    }
    public Value sub(final Value v) {
        if (v instanceof ValueLong) { return this.internalSub((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalSub((ValueReal) v); }
        return this.internalSub(v);
    }
    public Value mul(final Value v) {
        if (v instanceof ValueLong) { return this.internalMul((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalMul((ValueReal) v); }
        return this.internalMul(v);
    }
    public Value div(final Value v) {
        if (v instanceof ValueLong) { return this.internalDiv((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalDiv((ValueReal) v); }
        return this.internalDiv(v);
    }
    public Value mod(final Value v) {
        if (v instanceof ValueLong) { return this.internalMod((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalMod((ValueReal) v); }
        return this.internalMod(v);
    }

    public boolean not() { return ! isTrue(); }

    public boolean lt(final Value v) {
        if (v instanceof ValueLong) { return this.internalLt((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalLt((ValueReal) v); }
        return this.internalLt(v);
    }
    public boolean gt(final Value v) {
        if (v instanceof ValueLong) { return this.internalGt((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalGt((ValueReal) v); }
        return this.internalGt(v);
    }
    public boolean le(final Value v) {
        if (v instanceof ValueLong) { return this.internalLe((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalLe((ValueReal) v); }
        return this.internalLe(v);
    }
    public boolean ge(final Value v) {
        if (v instanceof ValueLong) { return this.internalGe((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalGe((ValueReal) v); }
        return this.internalGe(v);
    }
    public boolean eq(final Value v) {
        if (v instanceof ValueLong) { return this.internalEq((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalEq((ValueReal) v); }
        if (v instanceof ValueObject) { return this.internalEq((ValueObject) v); }
        return this.internalEq(v);
    }
    public boolean ne(final Value v) {
        if (v instanceof ValueLong) { return this.internalNe((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalNe((ValueReal) v); }
        if (v instanceof ValueObject) { return this.internalNe((ValueObject) v); }
        return this.internalNe(v);
    }


    // Internal Operators
    private Value internalAdd(final Value v) { assert DISABLE_ASSERT : "adding Unknown Value type: " + v; return this; }
    private Value internalAdd(final ValueLong v) { return Value.newValue(v.getLong() + this.value); }
    private Value internalAdd(final ValueReal v) { return Value.newValue(v.getReal() + this.value); }
    private Value internalAdd(final ValueString v) { return Value.newValue(v.getString() + this.value); }

    private Value internalSub(final Value v) { assert DISABLE_ASSERT : "subtracting Unknown Value type: " + v; return this; }
    private Value internalSub(final ValueLong v) { return Value.newValue(this.value - v.getLong()); }
    private Value internalSub(final ValueReal v) { return Value.newValue(this.value - v.getReal()); }

    private Value internalMul(final Value v) { assert DISABLE_ASSERT : "multplying Unknown Value type: " + v; return this; }
    private Value internalMul(final ValueLong v) { return Value.newValue(this.value * v.getLong()); }
    private Value internalMul(final ValueReal v) { return Value.newValue(this.value * v.getReal()); }

    private Value internalDiv(final Value v) { assert DISABLE_ASSERT : "dividing by Unknown Value type: " + v; return this; }
    private Value internalDiv(final ValueLong v) { return Value.newValue(this.value / v.getLong()); }
    private Value internalDiv(final ValueReal v) { return Value.newValue(this.value / v.getReal()); }

    private Value internalMod(final Value v) { assert DISABLE_ASSERT : "modulo-ing by Unknown Value type: " + v; return this; }
    private Value internalMod(final ValueLong v) { return Value.newValue(this.value % v.getLong()); }
    private Value internalMod(final ValueReal v) { return Value.newValue(this.value % v.getReal()); }

    // booleans
    private boolean internalLt(final Value v) { assert DISABLE_ASSERT : "comparing Unknown Value type: " + v; return false; }
    private boolean internalLt(final ValueLong v) { return this.value < v.getLong(); }
    private boolean internalLt(final ValueReal v) { return this.value < v.getReal(); }

    private boolean internalLe(final Value v) { assert DISABLE_ASSERT : "comparing Unknown Value type: " + v; return false; }
    private boolean internalLe(final ValueLong v) { return this.value <= v.getLong(); }
    private boolean internalLe(final ValueReal v) { return this.value <= v.getReal(); }

    private boolean internalGt(final Value v) { assert DISABLE_ASSERT : "comparing Unknown Value type: " + v; return false; }
    private boolean internalGt(final ValueLong v) { return this.value > v.getLong(); }
    private boolean internalGt(final ValueReal v) { return this.value > v.getReal(); }

    private boolean internalGe(final Value v) { assert DISABLE_ASSERT : "comparing Unknown Value type: " + v; return false; }
    private boolean internalGe(final ValueLong v) { return this.value >= v.getLong(); }
    private boolean internalGe(final ValueReal v) { return this.value >= v.getReal(); }

    private boolean internalEq(final Value v) { assert DISABLE_ASSERT : "comparing Unknown Value type: " + v; return false; }
    private boolean internalEq(final ValueLong v) { return this.value == v.getLong(); }
    private boolean internalEq(final ValueReal v) { return this.value == v.getReal(); }
    private boolean internalEq(final ValueObject v) { return this.isTrue() == v.isTrue(); } // allows (obj == 0.0)

    private boolean internalNe(final Value v) { assert DISABLE_ASSERT : "comparing Unknown Value type: " + v; return false; }
    private boolean internalNe(final ValueLong v) { return this.value != v.getLong(); }
    private boolean internalNe(final ValueReal v) { return this.value != v.getReal(); }
    private boolean internalNe(final ValueObject v) { return this.isTrue() != v.isTrue(); } // allows (obj != 0.0)

}

