package at.oefai.aaa.agent.jam.types;

import java.io.Serializable;

/**
 * Represents a built-in JAM String data-type.
 * @author Marc Huber
 * @author Jaeho Lee
 */
public class ValueString extends Value implements Serializable {
    private String value = null;

    ValueString(final String s) { this.value = s; }

    Value copy() { return new ValueString(this.value); }

    // Member functions

    public boolean isTrue() { return this.value.length() > 0; }

    public long getLong() { return 0; }
    public double getReal() { return 0.0; }
    public String getString() { return this.value; }
    public Object getObject() { return this.value; }

    public String toString() { return this.value; }

    // Operators

    public Value neg() { return copy(); } //no negating strings

    public Value add(final Value v) {
        if (v instanceof ValueLong) { return this.internalAdd((ValueLong) v); }
        if (v instanceof ValueReal) { return this.internalAdd((ValueReal) v); }
        if (v instanceof ValueString) { return this.internalAdd((ValueString) v); }
        return this.internalAdd(v);
    }
    public Value sub(final Value v) { assert DISABLE_ASSERT : "ValueString " + this.value + "can't be subtracted from"; return this; }
    public Value mul(final Value v) { assert DISABLE_ASSERT : "ValueString " + this.value + "can't be multiplied"; return this; }
    public Value div(final Value v) { assert DISABLE_ASSERT : "ValueString " + this.value + "can't be divided"; return this; }
    public Value mod(final Value v) { assert DISABLE_ASSERT : "ValueString " + this.value + "can't be modulo-ed"; return this; }

    public boolean not() { return ! isTrue(); }

    public boolean lt(final Value v) {
        if (v instanceof ValueString) { return this.internalLt((ValueString) v); }
        return this.internalLt(v);
    }
    public boolean gt(final Value v) {
        if (v instanceof ValueString) { return this.internalGt((ValueString) v); }
        return this.internalGt(v);
    }
    public boolean le(final Value v) {
        if (v instanceof ValueString) { return this.internalLe((ValueString) v); }
        return this.internalLe(v);
    }
    public boolean ge(final Value v) {
        if (v instanceof ValueString) { return this.internalGe((ValueString) v); }
        return this.internalGe(v);
    }
    public boolean eq(final Value v) {
        if (v instanceof ValueString) { return this.internalEq((ValueString) v); }
        return this.internalEq(v);
    }
    public boolean ne(final Value v) {
        if (v instanceof ValueString) { return this.internalNe((ValueString) v); }
        return this.internalNe(v);
    }


    // Internal Operators

    private Value internalAdd(final Value v) { assert DISABLE_ASSERT : "adding Unknown Value type: " + v; return this; }
    private Value internalAdd(final ValueLong v) { return Value.newValue(this.value + v.getLong()); }
    private Value internalAdd(final ValueReal v) { return Value.newValue(this.value + v.getReal()); }
    private Value internalAdd(final ValueString v) { return Value.newValue(this.value + v.getString()); }

    // booleans
    private boolean internalLt(final Value v) { assert DISABLE_ASSERT : "comparing non-ValueString: " + v; return false; }
    private boolean internalLt(final ValueString v) { return this.value.compareTo(v.getString()) < 0; }

    private boolean internalLe(final Value v) { assert DISABLE_ASSERT : "comparing non-ValueString: " + v; return false; }
    private boolean internalLe(final ValueString v) { return this.value.compareTo(v.getString()) <= 0; }

    private boolean internalGt(final Value v) { assert DISABLE_ASSERT : "comparing non-ValueString: " + v; return false; }
    private boolean internalGt(final ValueString v) { return this.value.compareTo(v.getString()) > 0; }

    private boolean internalGe(final Value v) { assert DISABLE_ASSERT : "comparing non-ValueString: " + v; return false; }
    private boolean internalGe(final ValueString v) { return this.value.compareTo(v.getString()) >= 0; }

    public boolean internalEq(final Value v) { assert DISABLE_ASSERT : "comparing non-ValueString: " + v; return false; }
    public boolean internalEq(final ValueString v) { return this.value.equals(v.getString()); }

    public boolean internalNe(final Value v) { assert DISABLE_ASSERT : "comparing non-ValueString: " + v; return false; }
    public boolean internalNe(final ValueString v) { return ! this.value.equals(v.getString()); }

}

