package at.oefai.aaa.agent.jam.types;

import java.io.Serializable;

/**
 * Represents a plan variable.
 * just contains a cached copy of the name under
 * which it is found in the symbol map it belongs to.
 * @author Marc Huber
 * @author Jaeho Lee
 * @author Stefan Rank
 */
public class Variable extends AbstractExpression implements Serializable {
    private final SymbolMap symbolMap; // variable owner
    private final String name; // local caching of the variable name

    /**
     * constructor only to be called from the SymbolMap (passed as parameter).
     * @param pSymbolMap the owner (and hopefully caller)
     * @param pName the name for this variable
     */
    Variable(final SymbolMap pSymbolMap, final String pName) {
        this.symbolMap = pSymbolMap;
        this.name = pName;
    }

    public final String getName() { return this.name; }

    public final Value eval(final Binding b) {
        return (b == null) ? Value.UNDEFINED : b.getValue(this);
    }

    /** Output information to the stream not in an in-line manner. */
    public final String verboseString(final Binding b) {
        return this.eval(b).verboseString(b);
    }

    /** Output information to the stream in an in-line manner. */
    public final String formattedString(final Binding b) {
        return this.eval(b).formattedString(b);
    }

}

