package at.oefai.aaa.animation;

import org.w3c.dom.svg.SVGPoint;
import org.w3c.dom.svg.SVGSVGElement;

/**
 * SVG anim for picking up an object which is on top of another one.
 * @author Stefan Rank
 */
public class TakeObjectFromObject extends BaseSVGAnim {
    private volatile boolean done = false;
    private SVGPoint dest = null;
    private SVGPoint coords = null;
    private String theObject = null;

    public BaseAnim newInstance() { return new TakeObjectFromObject(); }

    protected void doFirstTime() {
        assert (this.args.size() == 2) : "wrong arguments for TakeObjectFromObject";
        this.theObject = this.args.get(0).getString();
        String theFromObject = this.args.get(1).getString();
        SVGSVGElement eleActor = getNamedElement(this.actor);
        this.dest = getElementsPos(eleActor);
        if (this.dest == null) {
            assert false : "could not get agents (" + this.actor + ") position";
            this.done = true;
        } else {
            SVGSVGElement eleObject = getNamedElement(this.theObject);
            SVGPoint curr = getElementsDocumentPos(eleObject);
            if (curr == null) {
                assert false : "could not get objects (" + this.theObject + ") document relative position";
                this.done = true;
            } else {
                eleObject.getParentNode().removeChild(eleObject);
                // the object we took it from can be an actor too... so adjust holding
                adjustHolding(getNamedElement(theFromObject));
                setElementsPos(eleObject, curr);
                // add the object to the svg doc
                appendObjectToSVGDoc(eleObject);
            }
            // displace dest
            this.coords = getCoordsOfAgentsObject(eleObject, eleActor);
            addToSVGPoint(this.dest, this.coords);
        }
    }


    public boolean isFinished() {
        return this.done;
    }

    protected void doIt() {
        // move the object if it is still away from the destination
        SVGSVGElement eleObject = getNamedElement(this.theObject);
        SVGPoint curr = getElementsPos(eleObject);
        if (curr == null) {
            assert false : "could not get objects (" + this.theObject + ") position";
            this.done = true;
        } else {
            SVGPoint nextPos = getNextObjectPos(curr, this.dest);
            if (nextPos == this.dest) {
                SVGSVGElement eleAgent = getNamedElement(this.actor);
                // take out above element from where it is now:
                eleObject.getParentNode().removeChild(eleObject);
                setElementsPos(eleObject, this.coords);
                // put it under Agent in the tree (which puts it above on screen;-)
                eleAgent.appendChild(eleObject);
                adjustHolding(eleAgent);
                this.done = true;
            } else {
                setElementsPos(eleObject, nextPos);
            }
        }
    }

}
